#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import unittest
from datetime import timedelta

from cypresspoint.datatype import as_bool, reltime_to_timedelta


class TestRelativeTime(unittest.TestCase):

    def test_simple(self):
        f = reltime_to_timedelta
        td = timedelta

        self.assertEqual(f("9d"), td(days=9))
        self.assertEqual(f("2h"), td(hours=2))
        self.assertEqual(f("6m"), td(minutes=6))
        self.assertEqual(f("90s"), td(seconds=90))
        self.assertEqual(f("90"), td(seconds=90))

    @unittest.expectedFailure
    def test_combos(self):
        # These don't work either.  We don't need to support really fancy use
        # cases like this, but raise ValueError if the input isn't understood.
        f, td = reltime_to_timedelta, timedelta
        self.assertEqual(f("1d3h"), td(days=1, hours=3))
        self.assertEqual(f("2d2d"), td(days=4))

    @unittest.expectedFailure
    def test_invalid_values(self):
        # XXX: Known bug: https://github.com/Kintyre/cypresspoint/issues/1
        with self.assertRaises(ValueError):
            reltime_to_timedelta("no numbers here")

        with self.assertRaises(ValueError):
            reltime_to_timedelta("7Tb")

        with self.assertRaises(ValueError):
            reltime_to_timedelta("33 random words here")

    '''
    #
        def test_things_that_i_cant_explain(self):
            # Why is a tuple allowed here?   (not allowed explicitly in code)
            # Q:  caught by isinstance(, list) ?
    '''


class TestBoolConverter(unittest.TestCase):

    def test_bool_text(self):
        self.assertTrue(as_bool("True"))
        self.assertTrue(as_bool("true"))
        self.assertTrue(as_bool(True))
        self.assertFalse(as_bool("No"))
        self.assertFalse(as_bool(None))



from __future__ import unicode_literals

import re

from six import integer_types, string_types


def sanitize_fieldname(field):
    """ Remove unwanted characters from the provided field name.  The goal is to
    mimic the general field cleanup behavior of Splunk """
    # type: (str) -> (str)
    clean = re.sub(r'[^A-Za-z0-9_.{}\[\]-]', "_", field)
    # Remove leading/trailing underscores
    clean = clean.strip("_")
    return clean


def _dict_to_splunk_fields(obj, prefix=()):
    """
    Input:  Object   (dict, list, str/int/float)
    Output:  [  ( (name,name), value) ]

    Convention:  Arrays suffixed with "{}"
    """
    # type: ignore (Any[dict,list,str,int,float]) -> List[Tuple[Tuple[str]], Any]
    output = []
    if isinstance(obj, dict):
        for key, value in obj.items():
            key = sanitize_fieldname(key)
            output.extend(_dict_to_splunk_fields(value, prefix=prefix+(key,)))
    elif isinstance(obj, (list, list)):
        # Why (list, list) ^^^^^^^^^^ ???  Looks like a typo
        if prefix:
            prefix = prefix[:-1] + (prefix[-1] + "{}",)
            for item in obj:
                output.extend(_dict_to_splunk_fields(item, prefix=prefix))
    elif isinstance(obj, bool):
        output.append((prefix, "true" if obj else "false"))
    elif isinstance(obj, (string_types, integer_types, float)) or obj is None:
        output.append((prefix, obj))
    else:
        raise TypeError("Unsupported datatype {}".format(type(obj)))
    return output


def splunk_dot_notation(obj):
    """
    Convert json object (python dictionary) into a list of fields as Splunk does by default.
    Think of this as the same as calling Splunk's "spath" SPL command.
    """
    # type: (dict) -> dict
    d = {}
    if not isinstance(obj, dict):
        raise ValueError("Expected obj to be a dictionary, received {}"
                         .format(type(obj)))
    for field_pair, value in _dict_to_splunk_fields(obj):
        field_name = ".".join(field_pair)
        if field_name in d:
            if not isinstance(d[field_name], list):
                d[field_name] = [d[field_name]]
            d[field_name].append(value)
        else:
            d[field_name] = value
    return d



from __future__ import unicode_literals

import sys
from logging import Formatter, LogRecord, StreamHandler


class AlertActionFormatter(Formatter):
    """ A custom formatter for writting out Splunk's expected log messages to
    STDOUT for Modular alerts.

    Format:
        [LEVEL] [MESSAGE]
    """

    if sys.version_info < (3, 2):

        def format(self, record):
            # Python 2.7 workaround, as formatMessage() isn't used

            # type: (LogRecord) -> str
            record.message = record.getMessage()
            s = self.formatMessage(record)
            return s

    def formatMessage(self, record):
        # type: (LogRecord) -> str
        # Returns just the first line of message
        mesage_first_line = record.message.split("\n")[0]
        return "{} {}".format(record.levelname, mesage_first_line)

    def formatException(self, exec_info):
        """ Just suppress because any output here triggers a multiline event. """
        return ""


def add_simple_stderr_handler(logger, stream=None):
    handler = StreamHandler(stream)
    handler.setLevel(logger.level)
    handler.setFormatter(AlertActionFormatter())
    logger.addHandler(handler)
    return handler



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from cypresspoint.results import expand_vars


class TestExpandVars(unittest.TestCase):

    def test_simple_curly(self):
        style = "{}"
        out = expand_vars("hi there {name}", dict(name="bob", other="fancy"), style=style)
        self.assertEqual(out, "hi there bob")

        out = expand_vars("hi there {MISSING}", {}, style=style)
        self.assertEqual(out, "hi there ")

        out = expand_vars("test with {not a var}", {"not a var": "BLAH"}, style=style)
        self.assertEqual(out, "test with {not a var}")

        out = expand_vars(" LITERAL {} ", {}, style=style)
        self.assertEqual(out, " LITERAL {} ")

    def test_simple_curly(self):
        style = "$"
        out = expand_vars("hi there $name$", dict(name="bob", other="fancy"), style=style)
        self.assertEqual(out, "hi there bob")

        out = expand_vars("hi there $MISSING$", {}, style=style)
        self.assertEqual(out, "hi there ")

        out = expand_vars("test with $not a var$", {"not a var": "BLAH"}, style=style)
        self.assertEqual(out, "test with $not a var$")

        out = expand_vars(" LITERAL $$", {}, style=style)
        self.assertEqual(out, " LITERAL $")



"""
Simple checkpoint tracking helper class with dictionary-like access.  This is
used for modular inputs to track their state between executions.
"""
from __future__ import unicode_literals

import json
import os
from copy import deepcopy
from datetime import datetime, timedelta
from logging import getLogger

from six.moves.urllib.parse import quote

logger = getLogger("cypresspoint.checkpoint")


def json_dt_converter(o):
    if isinstance(o, datetime):
        return o.__str__()


class ModInputCheckpoint(object):
    NotSet = object()

    def __init__(self, checkpoint_dir, input_name):
        self.checkpoint_dir = checkpoint_dir
        self.input_name = input_name
        self.filename = self._safe_filename(checkpoint_dir, input_name)
        self.dump_after_updates = 10
        self._data = None
        self.default_item = KeyError
        self.updates = 0
        self._last_dump = None

    def __del__(self):
        # XXX: Review this.  What are the rules about raising an exception in __del__; no guarantees
        if self.updates > 0:
            raise Exception("You forgot to dump the checkpoint data.   Changes not saved!")

    def _safe_filename(self, checkpoint_dir, input_name):
        input_name = input_name.replace("://", "__")
        input_name = quote(input_name, safe="")
        return os.path.join(checkpoint_dir, input_name + ".json")

    def load(self, default=None):
        # XXX:  Support pulling in old checksum data from backup, IF primary gets corrupted.
        #       Shouldn't happen as we attempt to create the file safely, but weird things happen.
        cp_filename = self.filename
        logger.debug("Loading checkpoint data from %s", cp_filename)
        try:
            with open(cp_filename) as fp:
                self._data = json.load(fp) or {}
        except (IOError, ValueError):
            # If you see this message logged frequently, then there's an issue.
            logger.warning("[%s] Unable to process checkpoint file '%s'.  "
                           "This is normal for a new input.  Starting from scratch.",
                           self.input_name, cp_filename)
            if default:
                self._data = default
            else:
                self._data = {}

    def dump_on_interval(self, delta):
        """ Dump to disk if interval has elapsed even if dump_after_updates
        hasn't been reached.  This is helpful for long-lived modular inputs.

        A dump will also occur the very first time this function is called.
        """
        # type: (timedelta)
        now = datetime.now()
        if self._last_dump:
            if self.updates == 0:
                return False
            age = now - self._last_dump
            if age < delta:
                return False
        self.dump(_now=now)
        return True

    def dump(self, _now=None):
        # Safely write out checkpoint data; this should avoid MOST common issues
        cp_filename = self.filename

        cp_filename_bk = cp_filename + ".bak"
        cp_filename_new = "{}.new-{}".format(cp_filename, os.getpid())
        try:
            with open(cp_filename_new, "w") as fp:
                # Indentation makes this human readable, with minimal overhead
                json.dump(self._data, fp, indent=2, default=json_dt_converter)

            # Backup existing file (unless this is the first-time run)
            if os.path.isfile(cp_filename):
                # Remove old backup, if present
                if os.path.isfile(cp_filename_bk):
                    os.unlink(cp_filename_bk)
                # Move previous live file to "backup"
                os.rename(cp_filename, cp_filename_bk)

            # Move NEW checkpoint data (temporary file) into place
            os.rename(cp_filename_new, cp_filename)
            if self._data is not None:
                logger.info("Checkpoint data saved to %s.  %d entries",
                            cp_filename, len(self._data))
        except Exception:
            logger.exception("[%s] Failure while writing out checkpoint data.  "
                             "State not saved.", self.input_name)
            logger.warning("[%s] Emergency dump of checkpoint data to log:\n%s",
                           self.input_name, json.dumps(self._data), default=json_dt_converter)
        self.updates = 0
        self._last_dump = _now or datetime.now()

    def __getitem__(self, item):
        try:
            return self._data[item]
        except KeyError:
            if isinstance(self.default_item, Exception):
                raise self.default_item(item)
            return deepcopy(self.default_item)

    def get(self, item, default=NotSet):
        try:
            return self._data[item]
        except KeyError:
            if default is self.NotSet:
                return deepcopy(self.default_item)
            else:
                return default

    def __setitem__(self, key, value):
        self._data[key] = value
        self.updates += 1
        if self.updates >= self.dump_after_updates:
            self.dump()



# -*- coding: utf-8 -*-

"""Top-level package for Cypress Point."""

from __future__ import unicode_literals

__author__ = """Lowell Alleman"""
__email__ = 'lowell@kintyre.co'
__version__ = '0.6.0'


def setup_logging(log_file, debug=False):
    from logging import DEBUG, INFO, Formatter, getLogger
    from logging.handlers import TimedRotatingFileHandler
    log = getLogger()
    if debug:
        log_level = DEBUG
    else:
        log_level = INFO
    log.setLevel(log_level)
    logfile = TimedRotatingFileHandler(log_file, "d", 7, backupCount=10, encoding='utf-8')
    logfile.setLevel(log_level)
    # setup log file formatting:
    logfile.setFormatter(
        Formatter('%(asctime)s [%(process)d] %(levelname)-8s %(name)s:  %(message)s'))
    log.addHandler(logfile)
    return log



""" cypresspoint.results:  Tools for working with search results
"""

import gzip
import re
import sys

if sys.version_info < (3,):
    # For consistent unicode support
    from backports import csv
else:
    import csv

csv.field_size_limit(10485760)


_expand_styles = {
    "$": (re.compile(r'\$([\w._]*)\$'), "$"),
    "{}": (re.compile(r'\{([\w._-]*)\}'), "{}"),
}


def expand_vars(string, context, style="$", log=None):
    """ Expand vars in strings.   $var$
    Very quick-n-dirty!

    Note that expanding '$var$' format only works in result fields not in
    paramaters.  That is because Splunk already does $var$ expansion on it's on,
    and anything that doesn't match is replaced with an empty string.  You can
    of course use $result.fieldname$, but this only works on the first result
    row and then you don't need this function.

    Given this limitation, support for '{field}' variable expansion was added.
    Maybe someday additional field formatting options could be supported too,
    like what Python supports natively, but for now only simple varable
    substitution works.
    """
    regex, literal = _expand_styles[style]

    def rep(match_obj):
        var = match_obj.group(1)
        if var == "":
            # Replace "$$" with "$", or "{}"" with "{}"
            return literal
        else:
            return context.get(var)

    result = regex.sub(rep, string)
    if result != string:
        if log is callable:
            log("Expanded '{}' to '{}'".format(string, result))
    return result


def _decode_mv_field(field,
                     mv_field_re=re.compile(r'\$((?:\$\$|[^$])*)\$(?:;|$)')):
    return [m.replace("$$", "$") for m in mv_field_re.findall(field)]


def get_job_results(results_file, encoding="utf8"):
    """
    Read in results from a results file (results.csv.gz)
    Generate: dict
    """
    if not results_file.endswith(".csv.gz"):
        raise NotImplementedError("Expecting results file to be a .csv.gz file.")

    mv_fields = None
    with gzip.open(results_file, "rt", encoding=encoding) as stream:
        for row in csv.DictReader(stream):
            # Remove __mv_ fields; replacing the origion fields with lists of values, were necessary
            if mv_fields is None:
                mv_fields = [(f, f[5:]) for f in row if f.startswith("__mv_")]
            if mv_fields:
                for mv_field, field in mv_fields:
                    if row[mv_field]:
                        row[field] = _decode_mv_field(row[mv_field])
                    del row[mv_field]
            yield row



""" Helper functions for custom Splunk Search Commands
"""


def ensure_fields(results):
    """ Ensure that the first result has a placeholder key for *ALL* the fields """
    # type: ignore (List[dict]) - > List[dict]
    # XXX: Make this smarter by only holding a fix number of results before moving on.
    # E.g. Assume no new fields after n rows with no new fields encountered
    field_set = set()
    output = []
    for result in results:
        field_set.update(result.keys())
        output.append(result)
    if output:
        # Apply *all* fields to the first result; all other rows are left alone
        # Q: Does this screw up field order?  (Not sure it's preserved in any case)
        output[0] = {k: output[0].get(k, None) for k in field_set}
    return output



"""
This class makes creating a REST handler for Splunk easier.

The class will automatically call a function within the class based on the path and the method.

For example, if a GET call is made to the path "ping", then this class will call the function
get_ping().

Below is an example.

1) Create your REST handler class

This class needs to inherit from RESTHandler.

See below for an example that will return

    import logging
    import rest_handler

    class HelloWorldHandler(rest_handler.RESTHandler):
        def __init__(self, command_line, command_arg):
            super(HelloWorldHandler, self).__init__(command_line, command_arg, logger)

        def get_ping(self, request_info, **kwargs):
            return {
                'payload': 'This worked!', # Payload of the request.
                'status': 200 # HTTP status code
            }

2) Register the REST handler in restmap.conf

Below is an example that assumes the Python code above is in hello_world_rest_handler.py.

    [script:hello_world_rest_handler]
    match                 = /data/my_test
    script                = hello_world_rest_handler.py
    scripttype            = persist
    handler               = hello_world_rest_handler.HelloWorldHandler
    requireAuthentication = true
    output_modes          = json
    passPayload           = true
    passHttpHeaders       = true
    passHttpCookies       = true

This is licensed under the MIT license:

Copyright 2019 Luke Murphey

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

import json
import os
import re
import sys

from splunk.persistconn.application import PersistentServerConnectionApplication

if sys.platform == "win32":
    import msvcrt

    # Binary mode is required for persistent mode on Windows.
    msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
    msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
    msvcrt.setmode(sys.stderr.fileno(), os.O_BINARY)


class RequestInfo(object):
    """
    This represents the request.
    """

    def __init__(self, user, session_key, method, path, query, raw_args):
        self.user = user
        self.session_key = session_key
        self.method = method
        self.path = path
        self.query = query
        self.raw_args = raw_args

    def get_service(self, app=None):
        # type: (RequestInfo) -> splunklib.client.Session
        import splunklib.client
        from six.moves.urllib.parse import urlparse
        url = urlparse(self.raw_args["server"]["rest_uri"])
        ns = self.raw_args["ns"]
        kw = {}
        if "user" in ns:
            kw["owner"] = self.user
        service = splunklib.client.connect(
            host=url.hostname,
            port=url.port,
            scheme=url.scheme,
            # username=session["user"],  # If username is provided splunklib attempts to login()
            token=self.session_key,
            app=app or ns.get("app", None),
            autologin=False, **kw)
        return service


class RESTHandler(PersistentServerConnectionApplication):
    """
    This is a REST handler base-class that makes implementing a REST handler easier.

    This works by resolving a name based on the path in the HTTP request and calls it. This class
    will look for a function that includes the HTTP verb followed by the path.abs

    For example, if a GET request is made to the endpoint is executed with the path
    /lookup_edit/lookup_contents, then this class will attempt to run a function named
    get_lookup_contents(). Note that the root path of the REST handler is removed.

    If a POST request is made to the endpoint is executed with the path
    /lookup_edit/lookup_contents, the this class will attempt to execute post_lookup_contents().

    The arguments to the function will be the following:

      * request_info (an instance of RequestInfo)
      * keyword arguments (**kwargs)
    """

    def __init__(self, command_line, command_arg, logger=None):
        self.logger = logger
        PersistentServerConnectionApplication.__init__(self)

    @classmethod
    def get_function_signature(cls, method, path):
        """
        Get the function that should be called based on path and request method.
        """

        if len(path) > 0:
            return method + "_" + re.sub(r'[^a-zA-Z0-9_]', '_', path).lower()
        else:
            return method

    def render_json(self, data, response_code=200, headers=None):
        """
        Render the data as JSON
        """

        combined_headers = {
            'Content-Type': 'application/json'
        }

        if headers is not None:
            combined_headers.update(headers)

        return {
            'payload': json.dumps(data),
            'status': response_code,
            'headers': combined_headers,
        }

    def render_error_json(self, message, response_code=500):
        """
        Render an error to be returned to the client.
        """

        data = {
            'success': False,
            'message': message
        }

        return {
            'payload': json.dumps(data),
            'status': response_code,
            'headers': {
                'Content-Type': 'application/json'
            },
        }

    def get_forms_args_as_dict(self, form_args):

        post_arg_dict = {}

        for arg in form_args:
            name = arg[0]
            value = arg[1]

            post_arg_dict[name] = value

        return post_arg_dict

    def handle(self, in_string):
        try:

            self.logger.debug("Handling a request")

            # Parse the arguments
            args = self.parse_in_string(in_string)

            # Get the user information
            session_key = args['session']['authtoken']
            user = args['session']['user']

            # Get the method
            method = args['method']

            # Get the path and the args
            if 'path_info' in args:
                path = args['path_info']
            else:
                return {
                    'payload': 'No path was provided',
                    'status': 403
                }

            if method.lower() == 'post':
                # Load the parameters from the query
                query = args['query_parameters']

                if query is None:
                    query = {}

                # Apply the ones (if any) we got from the form
                query_form = self.get_forms_args_as_dict(args["form"])

                if query_form is not None:
                    query.update(query_form)
            else:
                query = args['query_parameters']

            # Make the request info object
            request_info = RequestInfo(user, session_key, method, path, query, args)

            # Get the function signature
            function_name = self.get_function_signature(method, path)

            try:
                function_to_call = getattr(self, function_name)
            except AttributeError:
                function_to_call = None

            # Try to run the function
            if function_to_call is not None:
                if self.logger is not None:
                    self.logger.debug("Executing function, name=%s", function_name)

                # Execute the function
                return function_to_call(request_info, **query)
            else:
                if self.logger is not None:
                    self.logger.warn("A request could not be executed since the associated "
                                     "function is missing, name=%s", function_name)

                return {
                    'payload': 'Path was not found',
                    'status': 404
                }
        except Exception as exception:
            if self.logger is not None:
                self.logger.exception("Failed to handle request due to an unhandled exception")

            return {
                'payload': str(exception),
                'status': 500
            }

    def get_ping(self, request_info, **kwargs):
        """
        Return a response indicating that the REST handler is online.
        """

        return {
            'payload': 'Online',
            'status': 200,
            'headers': {
                'Content-Type': 'text/plain'
            },
        }

    def post_ping(self, request_info, **kwargs):
        """
        Return a response indicating that the REST handler is online.
        """

        return self.get_ping(request_info, **kwargs)

    def head_ping(self, request_info, **kwargs):
        """
        Return a response indicating that the REST handler is online.
        """

        return self.get_ping(request_info, **kwargs)

    def convert_to_dict(self, query):
        """
        Create a dictionary containing the parameters.
        """
        parameters = {}

        for key, val in query:

            # If the key is already in the list, but the existing entry isn't a list then make the
            # existing entry a list and add thi one
            if key in parameters and not isinstance(parameters[key], list):
                parameters[key] = [parameters[key], val]

            # If the entry is already included as a list, then just add the entry
            elif key in parameters:
                parameters[key].append(val)

            # Otherwise, just add the entry
            else:
                parameters[key] = val

        return parameters

    def parse_in_string(self, in_string):
        """
        Parse the in_string
        """

        params = json.loads(in_string)

        params['method'] = params['method'].lower()

        params['form_parameters'] = self.convert_to_dict(params.get('form', []))
        params['query_parameters'] = self.convert_to_dict(params.get('query', []))

        return params



from __future__ import absolute_import

"""
Note that 'unicode_literals' is NOT used in by this module.  This is on
purpose.  Using the Python 2/3 default for 'str' works perfectly here.  This
limitation is due to same bad behavior in the Splunk SDK, IMHO.
"""

from logging import getLogger

# Apply fixes to splunk-sdk; this MUST occur before loading splunklib
import cypresspoint.monkeypatch  # noqa

from splunklib.modularinput import Script  # noqa

HIDDEN_PASSWORD = "*" * 8


class ScriptWithSimpleSecret(Script):
    """ Class that extends Splunk's default 'Script' that allows for very basic
    storage of a secret value.  Note that this techincally results in a
    race-condition where the unencrypted secret is exposed for some short period
    of time.  Therefore this shouldn't be used in high security scenarios or on
    servers with many snooping users, but for private use apps on a dedicated
    data onboarding forwarder, this approach is often "good enough".
    """
    secret_field = "secret"

    def __init__(self, *args, **kwargs):
        super(ScriptWithSimpleSecret, self).__init__(*args, **kwargs)
        self.logger = getLogger(self.__class__.__name__)

    def handle_secret(self, input_name, password, app=None):
        """
        Get, Set, or Update secret field as needed.

        This command will encrypt any clear-text password and mask it's value in
        ``inputs.conf``.  If a clear-text password is not given, then it's assumed
        to have been previously saved and will be loaded from ``passwords.conf``

        :param str input_name: Stanza in ``inputs.conf`` of the modular input
        :param str password: The recipient of the message
        :param app: The splunk app namesspace to use for REST interactions
                    against inputs and password endpoints
        :type app: str or None
        :return: the clear-text password
        :rtype: str
        """
        # type: (str, str, str) -> str
        if app:
            self.service.namespace.app["app"] = app
        field = self.secret_field
        logger = self.logger
        scheme, name = input_name.split("://", 1)
        pw_name = "{}:{}".format(scheme, name)
        pw = self.service.storage_passwords
        if password == HIDDEN_PASSWORD:
            logger.debug("Grabbing %s from password store!", field)
            return pw[pw_name].clear_password
        else:
            try:
                p = pw[pw_name]
                if p.clear_password != password:
                    self.logger.info("Updated encrypted %s for input %s", field, input_name)
                    p.update(password=password)
                else:
                    # If this happens frequently without a password change this indicates a problem.
                    self.logger.info("Password update encountered;  No change made.")
            except KeyError:
                logger.info("Converting clear-text %s in inputs.conf to encrypted %s "
                            "store for input %s", field, field, input_name)
                pw.create(password, name, scheme)
            # Clear existing password in the input
            inputs = self.service.confs["inputs"]
            stanza = inputs[input_name]
            stanza.update(api_key=HIDDEN_PASSWORD)
            logger.info("Updated inputs.conf [%s] %s=%s", input_name, field, HIDDEN_PASSWORD)
            return password



""" Hot in-place fixes for splunklib (Splunk's Python SDK)

NOTE: Simply importing this module will activate the monkeypatch.
"""

from __future__ import unicode_literals


def _monkey_patch_splunk_xml_parser():
    """ Return APP to know which app contains the inputs.conf stanza

    <stanza name="mod_input://MyInput" app="search">

    Hopefully this becomes unnecessary in the future:
    https://github.com/splunk/splunk-sdk-python/pull/371
    """
    # We store the 'app' in '__app' for later access.  This was chosen because
    # '__' is never used by apps, and it fits with other workarounds already
    # present in the splunk-sdk.

    import splunklib.modularinput.utils

    def parse_xml_data(parent_node, child_node_tag):
        from splunklib.modularinput.utils import parse_parameters
        data = {}
        for child in parent_node:
            if child.tag == child_node_tag:
                if child_node_tag == "stanza":
                    data[child.get("name")] = {}
                    # Added line here:
                    data[child.get("name")]["__app"] = child.get("app", None)
                    for param in child:
                        data[child.get("name")][param.get("name")] = parse_parameters(param)
            elif "item" == parent_node.tag:
                data[child.get("name")] = parse_parameters(child)
        return data

    splunklib.modularinput.utils.parse_xml_data = parse_xml_data

    import splunklib.modularinput.input_definition
    splunklib.modularinput.input_definition.parse_xml_data = parse_xml_data


_monkey_patch_splunk_xml_parser()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import unittest
from datetime import timedelta
from io import StringIO
from logging import DEBUG, INFO, Formatter, StreamHandler, getLogger
from logging.handlers import TimedRotatingFileHandler

from cypresspoint.log import AlertActionFormatter


class TestLogging(unittest.TestCase):

    def setup_logger(self, level, stream=None):
        root = getLogger()
        root.setLevel(level)
        if stream is None:
            stream = StringIO()
            self.stream = stream
        stream_handler = StreamHandler(stream)
        stream_handler.setLevel(level)
        # setup log file formatting:
        stream_handler.setFormatter(AlertActionFormatter())
        root.addHandler(stream_handler)
        return root

    def test_basic_formatting(self):
        logger = self.setup_logger(DEBUG)
        logger.error("Found half a worm in my apple")
        content = self.stream.getvalue()
        self.assertEqual(content, "ERROR Found half a worm in my apple\n")
        self.stream.seek(0)

        logger.warning("Found half a worm in my apple!")
        content = self.stream.getvalue()
        self.assertEqual(content, "WARNING Found half a worm in my apple!\n")

    def test_shorten_message(self):
        logger = self.setup_logger(DEBUG)
        logger.info("Here is a message\nthat goes across\nmultiple\nlines")
        content = self.stream.getvalue()
        self.assertEqual(content, "INFO Here is a message\n")

    def test_unhandled_exception(self):
        logger = self.setup_logger(DEBUG)
        try:
            raise ValueError("can't do it capt'n!")
        except:
            logger.exception("Enterprise down")
        content = self.stream.getvalue()
        self.assertEqual(content, "ERROR Enterprise down\n")



"""

"""

from __future__ import unicode_literals

import re
from datetime import timedelta


def reltime_to_timedelta(value):
    """Convert a relative time expression into a Python timedelta object.
    Only a subset of Splunk's relative time syntax is supported, but many simple
    expressions like ``7d`` (7 days), ``5m`` (5 mins), should just work.

    This does *not* support snapping with ``@``.
    Currently combining relative times addition (``+``) or subtraction (``-``)
    is not yet supported, but should be.

    :param value: Relative time expression
    :type value: str
    :return: python object representation of the given relative time
    :rtype: timedelta
    """
    pattern = re.compile(r"(\d+)([dhms]?)")
    suffix_map = {
        "s": "seconds",
        "m": "minutes",
        "h": "hours",
        "d": "days",
    }
    m = pattern.match(value)
    if m is None:
        raise ValueError("Unsupported span value: '{0}'  "
                         "Supports formats like '7d', '2h', and '15m'".format(value))
    v, suffix = m.groups()
    try:
        v = int(v)
    except ValueError:
        raise ValueError("Unsupported value: '{0}'".format(value))
    if not suffix:
        suffix = "s"
    td_arg = suffix_map[suffix]
    kwargs = {td_arg: v}
    return timedelta(**kwargs)


def as_bool(s):
    """Convert a boolean-like configuration field into a proper boolean.

    :param s: Input string containing some form of truthy value
    :type s: str
    :rtype: bool
    """
    if s is None or isinstance(s, bool):
        return s
    try:
        return s.lower()[0] in ("t", "y", "e", "1")
    except AttributeError as e:
        raise ValueError(e)



"""
A collection of Python 2/3 compatibility functions.

These should go away when Splunk 7.3 support is dropped.
"""


def dt_to_epoch(ts):
    """ Convert datetime to an Unix epoch value

    :param datetime ts: timestamp
    :rtype: int
    """
    try:
        # Python 3
        return int(ts.timestamp())
    except AttributeError:
        # XXX:  This isn't always as accurate if timezones are involved....
        return int(ts.strftime("%s"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import find_packages, setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.readlines()

setup_requirements = []

test_requirements = []

setup(
    author="Lowell Alleman",
    author_email='lowell@kintyre.co',
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*, !=3.6.*',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    description="Python helper bits for Splunk add-on development",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='cypresspoint',
    name='cypresspoint',
    packages=find_packages(include=['cypresspoint', 'cypresspoint.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Kintyre/cypresspoint',
    version='0.6.0',
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import shutil
import unittest
from tempfile import mkdtemp

from cypresspoint.checkpoint import ModInputCheckpoint


class TestCheckpoint(unittest.TestCase):

    def setUp(self):
        self.test_dir = mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def test_simple(self):
        MY_INPUT = "my_mod_input://stanza"
        # Create (first time)
        cp = ModInputCheckpoint(self.test_dir, MY_INPUT)
        cp.load()
        last_seen = cp.get("last_seen", "DEFAULT")
        self.assertEqual(last_seen, "DEFAULT")
        cp["last_seen"] = "UPDATED-VALUE"
        self.assertEqual(cp["last_seen"], "UPDATED-VALUE")
        self.assertFalse(os.path.isfile(cp.filename))
        cp.dump()
        self.assertTrue(os.path.isfile(cp.filename))
        del cp

        # Simulate second run
        cp = ModInputCheckpoint(self.test_dir, MY_INPUT)
        cp.load()
        self.assertEqual(cp["last_seen"], "UPDATED-VALUE")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import unittest

from six import PY3, text_type

from cypresspoint.spath import splunk_dot_notation


class TestSpath(unittest.TestCase):

    def test_simple_object(self):
        test = {
            "a": 1,
            "b": 2,
        }
        output = splunk_dot_notation(test)
        self.assertEqual(output, test)

    def test_invalid_field_name(self):
        f = splunk_dot_notation
        self.assertEqual(f({"_a": 1, "b": 2}), {"a": 1, "b": 2})
        self.assertEqual(f({"$junk": "apple"}), {"junk": "apple"})
        self.assertEqual(f({"ca$h": "apple"}), {"ca_h": "apple"})

        # Make sure values are unaffected
        self.assertEqual(f({"good": "$$!CRAZY#$"}), {"good": "$$!CRAZY#$"})

    def test_invalid_values(self):
        f = splunk_dot_notation

        with self.assertRaises(TypeError):
            # Sometimes this doesn't raise an exception???
            splunk_dot_notation({"field": tuple()})

        with self.assertRaises(TypeError):
            f({"field": set()})
        with self.assertRaises(TypeError):
            f({"field": object()})
        with self.assertRaises(TypeError):
            f({"field": object})

    def test_bad_input(self):
        with self.assertRaises(ValueError):
            splunk_dot_notation([{"a": 1}, {"a": 2}])

    '''
    #@unittest.expectedFailure
        def test_things_that_i_cant_explain(self):
            # Why is a tuple allowed here?   (not allowed explicitly in code)
            # Q:  caught by isinstance(, list) ?
    '''

    def test_unicode(self):
        # Reproduce bug found between Python 2 vs 3
        splunk_dot_notation({"a": text_type("value")})

    def test_bool_to_text(self):
        d = {
            "t": True,
            "f": False
        }
        self.assertEqual(splunk_dot_notation(d), {"t": "true", "f": "false"})

    @unittest.skipIf(PY3, "Long datatype doesn't exist in Python 3")
    def test_long(self):
        l = long(10000000000000000000000000000)
        self.assertEqual(splunk_dot_notation({"a": l}), {"a": l})

    def test_nested01(self):
        d = {
            "object": {"a": 1, "x": "two"},
            "array": ["1", "2", "3"]
        }
        e = {
            "object.a": 1,
            "object.x": "two",
            "array{}": ["1", "2", "3"]
        }
        output = splunk_dot_notation(d)
        self.assertEqual(output, e)

        d["array_of_objs"] = [
            {"a": 1, "z": 1},
            {"a": 2, "z": 2},
        ]
        e["array_of_objs{}.a"] = [1, 2]
        e["array_of_objs{}.z"] = [1, 2]
        output = splunk_dot_notation(d)
        self.assertEqual(output, e)

    def test_nested02(self):
        d = {
            "server": {
                "obj": [1, 2, 3]
            },
            "blah": {
                "a": 1,
                "b": 2,
                "c": [
                    {"a": 99},
                    {"a": 98}
                ],
            }
        }
        e = {
            "server.obj{}": [1, 2, 3],
            "blah.a": 1,
            "blah.b": 2,
            "blah.c{}.a": [99, 98],
        }
        output = splunk_dot_notation(d)
        self.assertDictEqual(output, e)
